@@ -317,7 +317,7 @@ class CrudManager(BaseManager):
 
     def _filter_kwargs(self, kwargs):
         """Drop null values and handle ids."""
-        for key, ref in six.iteritems(kwargs.copy()):
+        for key, ref in kwargs.copy().items():
             if ref is None:
                 kwargs.pop(key)
             else:
@@ -475,7 +475,7 @@ class Resource(object):
         return None
 
     def _add_details(self, info):
-        for (k, v) in six.iteritems(info):
+        for (k, v) in info.items():
             try:
                 setattr(self, k, v)
                 self._info[k] = v
@@ -497,7 +497,7 @@ class Resource(object):
     def get(self):
         """Support for lazy loading details.
 
-        Some clients, such as novaclient have the option to lazy load the
+        Some clients, such as watcherclient have the option to lazy load the
         details, details which can be loaded with this function.
         """
         # set_loaded() first ... so if we have to bail, we know we tried.
@@ -521,6 +521,9 @@ class Resource(object):
             return self.id == other.id
         return self._info == other._info
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def is_loaded(self):
         return self._loaded
 
