@@ -34,16 +34,15 @@ from six.moves import http_client
 import six.moves.urllib.parse as urlparse
 
 from watcherclient._i18n import _
+from watcherclient.common import api_versioning
 from watcherclient import exceptions
 
 
-# NOTE(deva): Record the latest version that this client was tested with.
-#             We still have a lot of work to do in the client to implement
-#             microversion support in the client properly! See
-#             http://specs.openstack.org/openstack/watcher-specs/specs/kilo/api-microversions.html # noqa
-#             for full details.
-DEFAULT_VER = '1.0'
-
+# Record the latest version that this client was tested with.
+DEFAULT_VER = '1.latest'
+# Minor version 4 for adding webhook API
+LAST_KNOWN_API_VERSION = 4
+LATEST_VERSION = '1.{}'.format(LAST_KNOWN_API_VERSION)
 
 LOG = logging.getLogger(__name__)
 USER_AGENT = 'python-watcherclient'
@@ -104,20 +103,8 @@ class VersionNegotiationMixin(object):
                 {'valid': ', '.join(API_VERSION_SELECTED_STATES),
                  'value': self.api_version_select_state})
         min_ver, max_ver = self._parse_version_headers(resp)
-        # NOTE: servers before commit 32fb6e99 did not return version headers
-        # on error, so we need to perform a GET to determine
-        # the supported version range
-        if not max_ver:
-            LOG.debug('No version header in response, requesting from server')
-            if self.os_watcher_api_version:
-                base_version = ("/v%s" %
-                                str(self.os_watcher_api_version).split('.')[0])
-            else:
-                base_version = API_VERSION
-            resp = self._make_simple_request(conn, 'GET', base_version)
-            min_ver, max_ver = self._parse_version_headers(resp)
         # If the user requested an explicit version or we have negotiated a
-        # version and still failing then error now.  The server could
+        # version and still failing then error now. The server could
         # support the version requested but the requested operation may not
         # be supported by the requested version.
         if self.api_version_select_state == 'user':
@@ -126,33 +113,33 @@ class VersionNegotiationMixin(object):
                   "server or the requested operation is not supported by the "
                   "requested version.  Supported version range is %(min)s to "
                   "%(max)s")
-                % {'req': self.os_watcher_api_version,
+                % {'req': self.os_infra_optim_api_version,
                    'min': min_ver, 'max': max_ver}))
         if self.api_version_select_state == 'negotiated':
             raise exceptions.UnsupportedVersion(textwrap.fill(
                 _("No API version was specified and the requested operation "
                   "was not supported by the client's negotiated API version "
                   "%(req)s.  Supported version range is: %(min)s to %(max)s")
-                % {'req': self.os_watcher_api_version,
+                % {'req': self.os_infra_optim_api_version,
                    'min': min_ver, 'max': max_ver}))
 
         negotiated_ver = str(
-            min(version.StrictVersion(self.os_watcher_api_version),
+            min(version.StrictVersion(self.os_infra_optim_api_version),
                 version.StrictVersion(max_ver)))
         if negotiated_ver < min_ver:
             negotiated_ver = min_ver
         # server handles microversions, but doesn't support
         # the requested version, so try a negotiated version
         self.api_version_select_state = 'negotiated'
-        self.os_watcher_api_version = negotiated_ver
+        self.os_infra_optim_api_version = negotiated_ver
         LOG.debug('Negotiated API version is %s', negotiated_ver)
 
         return negotiated_ver
 
     def _generic_parse_version_headers(self, accessor_func):
-        min_ver = accessor_func('X-OpenStack-Watcher-API-Minimum-Version',
+        min_ver = accessor_func('OpenStack-API-Minimum-Version',
                                 None)
-        max_ver = accessor_func('X-OpenStack-Watcher-API-Maximum-Version',
+        max_ver = accessor_func('OpenStack-API-Maximum-Version',
                                 None)
         return min_ver, max_ver
 
@@ -206,8 +193,8 @@ class HTTPClient(VersionNegotiationMixin):
         self.endpoint_trimmed = _trim_endpoint_api_version(endpoint)
         self.auth_token = kwargs.get('token')
         self.auth_ref = kwargs.get('auth_ref')
-        self.os_watcher_api_version = kwargs.get('os_watcher_api_version',
-                                                 DEFAULT_VER)
+        self.os_infra_optim_api_version = kwargs.get(
+            'os_infra_optim_api_version', DEFAULT_VER)
         self.api_version_select_state = kwargs.get(
             'api_version_select_state', 'default')
         self.conflict_max_retries = kwargs.pop('max_retries',
@@ -289,7 +276,7 @@ class HTTPClient(VersionNegotiationMixin):
         LOG.debug('\n'.join(dump))
 
     def _make_connection_url(self, url):
-        return urlparse.urljoin(self.endpoint_trimmed, url)
+        return '%s/%s' % (self.endpoint_trimmed.rstrip('/'), url.lstrip('/'))
 
     def _parse_version_headers(self, resp):
         return self._generic_parse_version_headers(resp.headers.get)
@@ -307,9 +294,15 @@ class HTTPClient(VersionNegotiationMixin):
         # Copy the kwargs so we can reuse the original in case of redirects
         kwargs['headers'] = copy.deepcopy(kwargs.get('headers', {}))
         kwargs['headers'].setdefault('User-Agent', USER_AGENT)
-        if self.os_watcher_api_version:
-            kwargs['headers'].setdefault('X-OpenStack-Watcher-API-Version',
-                                         self.os_watcher_api_version)
+        if self.os_infra_optim_api_version:
+            api_version = api_versioning.get_api_version(
+                self.os_infra_optim_api_version)
+            if api_version.is_latest():
+                api_version = api_versioning.get_api_version(
+                    LATEST_VERSION)
+            kwargs['headers'].setdefault(
+                'OpenStack-API-Version',
+                ' '.join(['infra-optim', api_version.get_string()]))
         if self.auth_token:
             kwargs['headers'].setdefault('X-Auth-Token', self.auth_token)
 
@@ -334,8 +327,8 @@ class HTTPClient(VersionNegotiationMixin):
 
             if resp.status_code == http_client.NOT_ACCEPTABLE:
                 negotiated_ver = self.negotiate_version(self.session, resp)
-                kwargs['headers']['X-OpenStack-Watcher-API-Version'] = (
-                    negotiated_ver)
+                kwargs['headers']['OpenStack-API-Version'] = (
+                    ' '.join(['infra-optim', negotiated_ver]))
                 return self._http_request(url, method, **kwargs)
 
         except requests.exceptions.RequestException as e:
@@ -354,7 +347,15 @@ class HTTPClient(VersionNegotiationMixin):
         # Read body into string if it isn't obviously image data
         body_str = None
         if resp.headers.get('Content-Type') != 'application/octet-stream':
-            body_str = ''.join([chunk for chunk in body_iter])
+            # decoding byte to string is necessary for Python 3 compatibility
+            # this issues has not been found with Python 3 unit tests
+            # because the test creates a fake http response of type str
+            # the if statement satisfies test (str) and real (bytes) behavior
+            body_list = [
+                chunk.decode("utf-8") if isinstance(chunk, bytes)
+                else chunk for chunk in body_iter
+            ]
+            body_str = ''.join(body_list)
             self.log_http_response(resp, body_str)
             body_iter = six.StringIO(body_str)
         else:
@@ -387,8 +388,8 @@ class HTTPClient(VersionNegotiationMixin):
         content_type = resp.headers.get('Content-Type')
 
         if (resp.status_code in (http_client.NO_CONTENT,
-                                 http_client.RESET_CONTENT)
-                or content_type is None):
+                                 http_client.RESET_CONTENT) or
+                content_type is None):
             return resp, list()
 
         if 'application/json' in content_type:
@@ -477,13 +478,13 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
     """HTTP client based on Keystone client session."""
 
     def __init__(self,
-                 os_watcher_api_version,
+                 os_infra_optim_api_version,
                  api_version_select_state,
                  max_retries,
                  retry_interval,
                  endpoint,
                  **kwargs):
-        self.os_watcher_api_version = os_watcher_api_version
+        self.os_infra_optim_api_version = os_infra_optim_api_version
         self.api_version_select_state = api_version_select_state
         self.conflict_max_retries = max_retries
         self.conflict_retry_interval = retry_interval
@@ -508,9 +509,16 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
                 _trim_endpoint_api_version(self.endpoint_override)
             )
 
-        if getattr(self, 'os_watcher_api_version', None):
-            kwargs['headers'].setdefault('X-OpenStack-Watcher-API-Version',
-                                         self.os_watcher_api_version)
+        if getattr(self, 'os_infra_optim_api_version', None):
+            api_version = api_versioning.get_api_version(
+                self.os_infra_optim_api_version)
+            if api_version.is_latest():
+                api_version = api_versioning.get_api_version(
+                    LATEST_VERSION)
+            kwargs['headers'].setdefault(
+                'OpenStack-API-Version',
+                ' '.join(['infra-optim',
+                          api_version.get_string()]))
 
         endpoint_filter = kwargs.setdefault('endpoint_filter', {})
         endpoint_filter.setdefault('interface', self.interface)
@@ -521,8 +529,8 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
                                     raise_exc=False, **kwargs)
         if resp.status_code == http_client.NOT_ACCEPTABLE:
             negotiated_ver = self.negotiate_version(self.session, resp)
-            kwargs['headers']['X-OpenStack-Watcher-API-Version'] = (
-                negotiated_ver)
+            kwargs['headers']['OpenStack-API-Version'] = (
+                ' '.join(['infra-optim', negotiated_ver]))
             return self._http_request(url, method, **kwargs)
         if resp.status_code >= http_client.BAD_REQUEST:
             error_json = _extract_error_json(resp.content)
@@ -574,7 +582,7 @@ def _construct_http_client(endpoint=None,
                            session=None,
                            token=None,
                            auth_ref=None,
-                           os_watcher_api_version=DEFAULT_VER,
+                           os_infra_optim_api_version=DEFAULT_VER,
                            api_version_select_state='default',
                            max_retries=DEFAULT_MAX_RETRIES,
                            retry_interval=DEFAULT_RETRY_INTERVAL,
@@ -605,27 +613,29 @@ def _construct_http_client(endpoint=None,
                         'the session to construct a client: %s',
                         ', '.join(dvars))
 
-        return SessionClient(session=session,
-                             os_watcher_api_version=os_watcher_api_version,
-                             api_version_select_state=api_version_select_state,
-                             max_retries=max_retries,
-                             retry_interval=retry_interval,
-                             endpoint=endpoint,
-                             **kwargs)
+        return SessionClient(
+            session=session,
+            os_infra_optim_api_version=os_infra_optim_api_version,
+            api_version_select_state=api_version_select_state,
+            max_retries=max_retries,
+            retry_interval=retry_interval,
+            endpoint=endpoint,
+            **kwargs)
     else:
         if kwargs:
             LOG.warning('The following arguments are being ignored when '
                         'constructing the client: %s', ', '.join(kwargs))
 
-        return HTTPClient(endpoint=endpoint,
-                          token=token,
-                          auth_ref=auth_ref,
-                          os_watcher_api_version=os_watcher_api_version,
-                          api_version_select_state=api_version_select_state,
-                          max_retries=max_retries,
-                          retry_interval=retry_interval,
-                          timeout=timeout,
-                          ca_file=ca_file,
-                          cert_file=cert_file,
-                          key_file=key_file,
-                          insecure=insecure)
+        return HTTPClient(
+            endpoint=endpoint,
+            token=token,
+            auth_ref=auth_ref,
+            os_infra_optim_api_version=os_infra_optim_api_version,
+            api_version_select_state=api_version_select_state,
+            max_retries=max_retries,
+            retry_interval=retry_interval,
+            timeout=timeout,
+            ca_file=ca_file,
+            cert_file=cert_file,
+            key_file=key_file,
+            insecure=insecure)
